package gui.layout.percent2;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.util.Hashtable;

public class AnchorPercentLayout implements LayoutManager2{

	private Hashtable<Component, APConstraint> constraints;
	private Dimension minSize;
	
	public AnchorPercentLayout() {
		this(new Dimension(0, 0));
	}
	
	public AnchorPercentLayout(Dimension minSize) {
		constraints=new Hashtable<>();
		this.minSize=minSize;
	}
	
	@Override
	public void addLayoutComponent(String name, Component comp) {
	}

	@Override
	public void removeLayoutComponent(Component comp) {
		
	}

	@Override
	public Dimension preferredLayoutSize(Container parent) {
		return parent.getSize();
	}

	@Override
	public Dimension minimumLayoutSize(Container parent) {
		return minSize;
	}

	@Override
	public void layoutContainer(Container target) {
		synchronized (target.getTreeLock()) {
			int count=target.getComponentCount();
			
			Insets border=target.getInsets();
			int width = target.getWidth() - border.left-border.right;
			int height = target.getHeight() - border.top-border.bottom;
			
			for (int i=0;i<count;i++) {
				Component comp = target.getComponent(i);
				APConstraint c = constraints.get(comp);
				if(c!=null) {
					int x=0,y=0,w=0,h=0;
					
					
					if (c.left()) {
						x=c.getLeft(width);
								
						if (c.right()) {
							w=width-x-c.getRight(width);
						} else if (c.width()){
							w=c.getWidth(width);
						} else {
							w=comp.getMinimumSize().width;
						}
					} else {
						if (c.width())
							w=c.getWidth(width);
						else
							w=comp.getMinimumSize().width;
						
						if (c.right()) {
							x = width- c.getRight(width) - w; 
						} else {
							x = (width-w)/2;
						}
						
					}
					
					if (c.top()) {
						y=c.getTop(height);
								
						if (c.bottom()) {
							h=height-y-c.getBottom(height);
						} else if (c.height()){
							h=c.getHeight(height);
						} else {
							h=comp.getMinimumSize().height;
						}
					} else {
						if (c.height())
							h=c.getHeight(height);
						else
							h=comp.getMinimumSize().height;
						
						if (c.bottom()) {
							y = height- c.getBottom(height) - h; 
						} else {
							y = (height-h)/2;
						}
					}
					
					comp.setBounds(x+border.left, y+border.top, w, h);
				}
			}
		}
	}

	
	@Override
	public void addLayoutComponent(Component c, Object constraint) {
		synchronized (c.getTreeLock()) {
			if (constraint instanceof APConstraint) {
				APConstraint cons=(APConstraint) constraint;
				constraints.put(c, cons);
			}
		}
	}

	@Override
	public Dimension maximumLayoutSize(Container target) {
		return target.getMaximumSize();
	}

	@Override
	public float getLayoutAlignmentX(Container target) {
		return 0;
	}

	@Override
	public float getLayoutAlignmentY(Container target) {
		return 0;
	}

	@Override
	public void invalidateLayout(Container target) {
		// TODO Auto-generated method stub
		
	}

}
